import { MessageTransmissionTransmitter } from "./msg.transmission.transmitter";
import { MessageTransmissionReceiver } from "./msg.transmission.receiver";
import { AdapterManager } from "../adapters/adapter.manager";
import { v4 as uuidv4 } from 'uuid'
import { MessageTransmissionRequestResponse } from "./msg.transmission.request-response";
import { Subject } from "rxjs";
import ConsoleLogger from "../utils/log.utils";
import { GeneralEvent, MessageReceiverInterface, MessageTransmitterInterface, TransmissionProfile } from "../interface/interface";
import { MessageTransmissionManagerBase } from "../base/msg.transmission.manager.base";

export class MessageTransmissionManager extends MessageTransmissionManagerBase {
    private console: ConsoleLogger = new ConsoleLogger(`MessageTransmissionManager`, ['managers'])

    constructor(event: Subject<GeneralEvent<any>>, browserEnv?: boolean) {
        super()
        if (browserEnv) this.browserEnv = browserEnv
        this.console.log({ message: `Contructing self... ${this.browserEnv ? `is receiving end` : `is not browser env`}` })
        this.event = event
        // Instantiate adapter manager
        this.adapterManager = new AdapterManager(event, this.browserEnv)
    }

    // for now these fuctions ain't pure, cuz messageTransmissionManager needs to keep a record of these tranmission instances as well for safe keeping.
    public getTransmitter(profile: TransmissionProfile): MessageTransmitterInterface {
        let transmitterInstance = new MessageTransmissionTransmitter(profile, this.adapterManager)
        this.tranmissionRef.push(transmitterInstance)
        return transmitterInstance
    }

    public getReceiver(profile: TransmissionProfile): MessageReceiverInterface {
        let receiverInstance = new MessageTransmissionReceiver(profile, this.adapterManager)
        this.tranmissionRef.push(receiverInstance)
        return receiverInstance
    }

    public getRequestResponse(profile: TransmissionProfile, transmitterInstance: MessageTransmitterInterface, receiverInstance: MessageReceiverInterface): MessageTransmissionRequestResponse {
        return new MessageTransmissionRequestResponse(profile, transmitterInstance, receiverInstance)
    }

}


